/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
// summary:
//		A widget that provides for display of both title and primary text in one grid cell.
//		Essentially a wrapper for the HTML and/or editor widgets that deal with those two
//		attributes.  Adds additional functionality to allow fields to be added/removed.

dojo.provide("com.ibm.rdm.web.artifactgrid.ContentCellWidget");

dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.project.EditManager");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedResourceLoader");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.ext.ArtifactExtension");
dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
dojo.require("dojox.layout.ResizeHandle");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.util.bidiutils"); //bidi
dojo.require("com.ibm.rdm.web.editor.EditorConstants");
dojo.require("com.ibm.rdm.web.util.vvcutils");
dojo.require("com.ibm.rdm.web.server.Headers");

dojo.requireLocalization("com.ibm.rdm.web.artifactgrid", "ContentCellWidget");

var DomUtils = com.ibm.rdm.web.util.domutils;
var EditManager = com.ibm.rdm.web.project.EditManager;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var EmbeddedResourceLoader = com.ibm.rdm.web.viewer.EmbeddedResourceLoader;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var MimeTypeRegistry = com.ibm.rdm.web.common.MimeTypeRegistry;
var ResourceManager = com.ibm.rdm.web.model.resources.ResourceManager;
var ArtifactExtension = com.ibm.rdm.web.ext.ArtifactExtension;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var BidiUtils = com.ibm.rdm.web.util.bidiutils;  //bidi
var EditorConstants = com.ibm.rdm.web.editor.EditorConstants;
var VvcUtils = com.ibm.rdm.web.util.vvcutils;
var Headers = com.ibm.rdm.web.server.Headers;

var INTERNAL_NAVIGATION_TABINDEX = "-2";

dojo.declare("com.ibm.rdm.web.artifactgrid.ContentCellWidget", [dijit._Widget, dijit._Templated], {
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "artifactgrid/templates/ContentCellWidget.html"),
	
	// Attach points
	_titleNode: null,
	_primaryTextNode: null,
	_addFieldAnchor: null,
	_overlayButtonContainerNode: null,
	
	// public fields - set on construction
	resource: null,		// set on construction
	artifactURI: null,	
	lockingURI: null,	//The uri to obtain a lock on
	isHeading: false,	// set true if the artifact is a heading.
	sectionNumber: null,	// The section number - only used if isHeading = true.
	depth: 1,				// Depth in the module
	titleEditable: false,		// set true to allow title to be edited.
	primaryTextEditable: false,	// set true to allow primary text to be edited.
	grid: null,
	columnId: null,				// Id of the column hosting this widget.
	rowKey:	null,				// Key of the row hosting this widget.
	editorCallbacks: null,		// A hash of callbacks that can be connected to editor hotkeys.

	// private fields
	// indicates which fields are currently on display
	_showingTitle: false,
	_showingPrimaryText: false,
	_messages: null,
	// If there is a heading, this is a reference to its <h#> node.
	_headingNode: null,
	// array of widgets that need to be destroyed when this one is destroyed.
	_widgetsToDestroy: null,
	
	//Track callback if the get lock fails
	_lockFailCallback: null,
	
	// Set true when we have an active editor on the primary text.
	_primaryTextUnderEdit: false,
	
	// Special message to display in primary text if the row is invalid
	_invalidArtifactRowMsg: null,
	
	postCreate: function() {
		this.inherited(arguments);
		
		this.titleEditable = this.params.titleEditable;
		this.primaryTextEditable = this.params.primaryTextEditable;
		
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.artifactgrid", "ContentCellWidget");
		this._invalidArtifactRowMsg = dojo.i18n.getLocalization("com.ibm.rdm.web.grid", "Module").invalidArtifactRow;
		
		this._show();
		
		this.connect(this.domNode, "onclick", this._handleClick);
	},
	
	uninitialize: function() {
		if (this._widgetsToDestroy) {
			for (var i = 0 ; i < this._widgetsToDestroy.length ; i++) {
				var widget = this._widgetsToDestroy[i];
				if (widget && widget.destroyRecursive) {
					widget.destroyRecursive();
				}
			}
		}
		this._widgetsToDestroy = null;
	},
	
	invokeEdit: function(eventNode){
		// summary:
		//		Starts edit.
		// eventNode:\
		//		The node on which an event occurred to start edit.  If present may be used
		//		to influence which field within the editor should be given focus initially.
        if (!this.isNonTextualArtifact) {
    		if (this.displayEditNameField){
    			this._startTitleEdit();
    		}
    		if (this.displayEditTextField){
    			this._startPrimaryTextEdit();
    		}
    		
    		return true;
		}
	},
	
	updateHeadingStyling: function(isHeading, depth, sectionNumber) {
		
		this.sectionNumber = sectionNumber;
		if (this._sectionNumberSpan) {
			this._sectionNumberSpan.innerHTML = sectionNumber;
		}
		
		if (isHeading === this.isHeading && depth === this.depth) {
			return;
		}
		
		this.depth = depth;
		this.isHeading = isHeading;
		

		if (!this._primaryTextUnderEdit) {
			// no editor open - we can just redraw everything
			this._refresh();
		} else {
			// editor open - we must try to change things in-situ on the editor.
			this._setPrimaryTextEditorStyle();
		}
	},
	
	setIsHeading: function(isHeading){
		this.isHeading = isHeading;
		if (this._primaryTextUnderEdit) {
			// we have an active editor, so we must not refresh as that would lose and re-create the editor
			this._setPrimaryTextEditorStyle();
		} else {
			// refresh to make sure everything displays just right.
			this._refresh();
		}
	},
	
	focus: function() {
		// summary:
		// 		Set focus to the editor within this widget.
		//		We should only be asked to focus when the editor is active
		// returns:
		//		true if focus was taken.
		var primaryTextURI = RRMTypes.getPrimaryTextURI(this.projectURI);
		var editorWidget = EditManager.getExistingEditor(this.projectURI, this.artifactURI, primaryTextURI);
		if (editorWidget) {
			editorWidget.focus();
			return true;
		}
	},
	
	startIntraCellNavigation: function(endIntraCellNavigationFunction) {
		// summary:
		//		Called to instruct the widget to start keyboard navigation within its content.  This allows
		//		keyboard navigation through the links within the text content.
		// endIntraCellNavigationFunction:
		//		Callback to call when keyboard focus is to be passed back to the containing grid.
		
		// Enable navigation by making a new widget that wraps the content and manages the navigation. 
		var navigationWidget = new com.ibm.rdm.web.artifactgrid.LinkNavigationWidget({
			onEndNavigation: endIntraCellNavigationFunction,
			focusNodeQuery: "[tabindex="+INTERNAL_NAVIGATION_TABINDEX+"]"
		}, this._primaryTextNode);
		if (navigationWidget.noNavigationRequired()) {
			// the widget has no work to do - get rid of it immediately to avoid having it
			// catch keypresses.
			navigationWidget.destroyRecursive(/*preserveDOM*/true);
		}
	},
	
	_resetDisplay: function() {
		// summary:
		//		Sets the DOM back to its default state, ready for it to
		//		be re-rendered using _show.
		
		// Move the title and primary text nodes to their default location, so they are not deleted
		// if we create a heading.  We may be doing a refresh, so one or other of them
		// may be under the heading node which we are going to delete.
		
//		dojo.place(this._titleNode, this.domNode);
//		dojo.removeClass(this._titleNode, "headingText");
//		this._removeHeadingMargin(this._titleNode);
		
		dojo.place(this._primaryTextNode, this.domNode);
		dojo.removeClass(this._primaryTextNode, "headingText");
		this._removeHeadingMargin(this._primaryTextNode);
		
		// remove any previous heading node
		if (this._headingNode && this._headingNode.parentNode) {
			this.domNode.removeChild(this._headingNode);
			this._headingNode = null;
		}
	},
	
	_show: function() {
		// summary:
		//		Displays the fields, positioning and styling them, and
		//		making appropriate ones editors.
		this._displayFields();

		// Note: This setTimeout is needed to avoid having the row that's
		// now going under edit temporarily show as very short before growing
		// tall again to display the editor.
		window.setTimeout(dojo.hitch(this, this._displayEditors), 0);
	},
	
	_refresh: function() {
		// summary:
		//		Redraws the display.  Used when an internal event requires
		//		the display to be refreshed.
		this._resetDisplay();
		this._show();
	},
	
	// Adds the necessary fields to the display.
	_displayFields: function(){
		if (!this.resource.isStub() && !this.resource.getID())
		{
			this.titleEditable = false;
			this.primaryTextEditable = false;
	
			// Display invalid artifact row message in primary text cell
			var spanNode = dojo.create('span', {"class" : "invalid-row-primary-text-msg", innerHTML: this._invalidArtifactRowMsg});
			dojo.place(spanNode, this._primaryTextNode, "first");
			
			return;
		}
		
		//is title derived from text
		//var hasServerGeneratedTitle = (this.resource.hasServerGeneratedTitle && this.resource.hasServerGeneratedTitle());
		var wrappedResourceURI = (this.resource.getWrappedResourceURI && this.resource.getWrappedResourceURI());
		var isRrcxArtifact = ArtifactExtension.isExtendedType(this.resource.getMimeType(), this._projectURI);
		var isDiagramArtifact = this.resource.getEntityType() === RRMTypes.getArtifactFormats(this.resource.getProjectURI()).Diagram;
		var isNonTextualArtifact = wrappedResourceURI || isRrcxArtifact || isDiagramArtifact;
		var showTitle, showPrimaryText;
		this.isNonTextualArtifact = isNonTextualArtifact;
		// Get title. Temporarily treat a single dash in title as an empty title - until we have a proper way
		// of knowing that a title has been calculated.
		var title = this.resource.getTitle();
		if (!title) {
			title = "";
		}
		
		// Get primary text
		// If there is a title and the primary text contains just whitespace, treat it as if the primary text
		// is empty. 
		var primaryText = this.resource.getPrimaryHtmlText();
		var cleanedPT = DomUtils.cleanHTMLString(primaryText);
		cleanedPT = dojo.trim(cleanedPT);
		// 62982 - if the content is empty save for an embed - don't clean it
		// 62499 - or indeed an img...
		if (!cleanedPT && (primaryText.indexOf("class=\""+EditorConstants.RICH_TEXT_EMBED_CLASS+"\"") === -1) && (primaryText.indexOf("img") === -1) && (primaryText.indexOf("IMG") === -1)) {
			primaryText = "";
		}
		
		// show or hide button that adds missing field.
		this.showAddTextButton = false;
		this.showAddNameButton = false;
		this.displayEditNameField = false;
		this.displayEditTextField = true;
		
		showTitle = false;
		showPrimaryText = true;
//		if (this.isHeading) {
//			if (!hasServerGeneratedTitle && primaryText) {
//			}
//			else if (!hasServerGeneratedTitle && isNonTextualArtifact) {
//			}
//			else if (hasServerGeneratedTitle && primaryText) {
//				showPrimaryText = false;
//			}
//			else if ((!hasServerGeneratedTitle && !primaryText) || 
//				(!isNonTextualArtifact && !title && !primaryText)) { //handle edge case
//				showPrimaryText = false;
//				this.showAddTextButton = true && this.primaryTextEditable;
//				this.displayEditTextField = false;
//			}
//		}
//		else {
//			if (!hasServerGeneratedTitle && primaryText) {
//			}
//			else if (!hasServerGeneratedTitle && isNonTextualArtifact) {
//			}
//			else if ((hasServerGeneratedTitle && primaryText) || 
//				(!isNonTextualArtifact && !title && !primaryText)) { //handle edge case
//				showTitle = false;
//				this.showAddNameButton = true && this.titleEditable;
//				this.displayEditNameField = false;
//			}
//			else if (!hasServerGeneratedTitle && !primaryText) {
//				showPrimaryText = false;
//			}
//		}
		//never allow to edit text field for non-text artifacts
		if (isNonTextualArtifact) {
			this.displayEditTextField = false;
			this.primaryTextEditable = false;
		}
		
		// Hide or display heading and primary text, depending on which is set, and upon
		// configuration of the widget.  Set some flags indicating which to display
		var attributeURI = RRMTypes.getWellKnownAttributeURI(this.projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title);
		var titleUnderEdit = EditManager.isUnderEdit(this.projectURI, this.artifactURI, attributeURI);
		attributeURI = RRMTypes.getPrimaryTextURI(this.projectURI);
		var primaryTextUnderEdit = EditManager.isUnderEdit(this.projectURI, this.artifactURI, attributeURI);
		

		// Always show a field if it is under edit
		showTitle = showTitle || titleUnderEdit;
		showPrimaryText = showPrimaryText || primaryTextUnderEdit;
		
		// keep the state of what we are displaying.
		this._showingTitle = showTitle;
		this._showingPrimaryText = showPrimaryText;
		
//		if (showTitle) {
//			dojo.place(document.createTextNode(title), this._titleNode, "only");
//		}
		
		if (wrappedResourceURI) {
			// it's a wrapped resource, see _getWrappedResourceDisplayValue for what we put
			// in the cell.
			
			var contentNode = null;
			var wrappedResourceRevisionURI = null;
			if (MimeTypeRegistry.isImage(this.resource.getWrappedResourceMimeType())) {
				// Embedded Resource Loader gets run over the cell and will expand
				// out with the correct revision.
				var wrappedResourceRevisionURI = this.resource.getAttribute(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.WrappedResourceRevisionURI);
				if (!wrappedResourceRevisionURI) {
					wrappedResourceURI = wrappedResourceURI + "?accept=none&private";
				}
				contentNode = this._createImg(wrappedResourceURI, wrappedResourceRevisionURI);
			} else {
				var resourceURI = this.resource.resourceURI;
				contentNode = dojo.create("a", {
					"href": resourceURI,
					"class": "embedded"
				});
			}
			
			dojo.place(contentNode, this._primaryTextNode, "only");
			dojo.style(this._primaryTextNode, "overflowX", "auto");
			
			if (!wrappedResourceRevisionURI) {
				this.loadEmbeddedResources(false);
			}
		} else if (isDiagramArtifact) {
			var resourceURI = this.resource.resourceURI;
			contentNode = dojo.create("a", {
				"href": resourceURI,
				"class": "embedded"
			});
			dojo.place(contentNode, this._primaryTextNode, "only");
			dojo.style(this._primaryTextNode, "overflowX", "auto");
			this.loadEmbeddedResources(false);
		} else if (isRrcxArtifact) {
			// it's an extended (RRCx type), here we can use the logic already
			// avaialble for embedded RRCx resources in  a rich text document,
			// just add an <a href="..." class="embedded"> into the cell
			// and let the embedded resource loader worry about generating the correct
			// URL for the convertor. 

			
			var resourceURI = this.resource.resourceURI;
			dojo.place(dojo.create("a", {
				"href": resourceURI,
				"class": "embedded"
			}), this._primaryTextNode, "only");

			dojo.addClass(this._primaryTextNode, "rrcxContent");
			this.loadEmbeddedResources(true);
		}
		else if (showPrimaryText){
			this._primaryTextNode.innerHTML = primaryText;
			
			dojo.query("a", this._primaryTextNode).forEach(function (node) {
				if (!dojo.hasClass(node, "embedded")) {
					
					// Because ID's are only unique per artifact, we
					// need to make sure Dojo does not try to use that
					// ID as the widget ID as it may well throw an error
					// when it finds that a widget with that ID is already
					// registered.
					var idAttrVal = dojo.attr(node, "id");
					if (idAttrVal) {
						dojo.removeAttr(node, "id");
					}
					
					
					var uri = dojo.attr(node, "href");
					if (uri) {
						// The user may have pasted an internal URI links, such as permaLinkURI for an artifact, a history
    					// or baseline revision.  In this case, we do not want to create an internal resource presentation
    					// link.  Just let the href stand as it is.  
    					this._addDestroy(ResourceLinksHandler.createLink({
    						uri: uri,
    						node: node,
    						retainLinkText: true,
    						returnWidget: true
    					}));
					}
					
					// Prevent links from interfering with tabbing through the grid, and make
					// sure they can be found for intra-cell navigation.
					dojo.attr(node, "tabIndex", INTERNAL_NAVIGATION_TABINDEX);
					
					if (idAttrVal) {
						dojo.attr(node, "id", idAttrVal);
					}
				}
			}, this);
			
			this.loadEmbeddedResources();
		}
		
		if (this.expandable) {
		   dojo.addClass(this.domNode, "expandable");
		}
		
		// Now build it all with the appropriate things in a heading if necessary
		
		var twistie;
		if (this.isHeading) {
			// Create a heading node that contains the section number
			var depth = this.depth;
			var headingTag = "h" + ((depth < 6) ? depth : 6);
			var headingNode = dojo.create(headingTag, {"class": "moduleHeading"}, this.domNode, "first");
			
			this.sectionNumberWrapper = dojo.create("div", {"class": "sectionNumber"}, headingNode);

			// Add a tooltip if section number suggests module has poor structure.
			// TODO: remove '-' test when section numbers contain parentheses.
			if (this.sectionNumber && ((this.sectionNumber.indexOf('(') >= 0) || (this.sectionNumber.indexOf('-') >= 0))) {
				dojo.attr(this.sectionNumberWrapper, "title", this._messages.parenthesesTip);
			} 
			
			
			
			dojo.place(this._createSectionSpan(), this.sectionNumberWrapper, "only");
			
			twistie = dojo.create("div", {"class": "twistie"}, this.sectionNumberWrapper);

			if (this.collapsed) {
				dojo.addClass(this.domNode, "collapsed");
			}
			
	
			// Move either the primary text or the title inside the heading
			if (!isNonTextualArtifact) {
				dojo.place(this._primaryTextNode, headingNode);
				dojo.addClass(this._primaryTextNode, "headingText");
				dojo.style(this.domNode, "visibility", "hidden");
				this._setHeadingMargin(this.sectionNumberWrapper, this._primaryTextNode);				
			}
			
			this.connect(this.sectionNumberWrapper, "onclick", this._toggleChildren);
			
			
			// remember the heading node, for when we need to redraw (see above)
			this._headingNode = headingNode;
		} else {
			twistie = dojo.create("div", {"class": "twistie"}, this.domNode);
			this.connect(twistie, "onclick", this._toggleChildren);
		}
		
		if (this._twistie) {
			dojo.destroy(this._twistie);
		}
		
		this._twistie = twistie;
		
		
		
//		dojo.style(this._titleNode, "display", showTitle?"block":"none");
		dojo.style(this._primaryTextNode, "display", showPrimaryText?"block":"none");
		// Make sure we have a reasonable click target in the case where primary text is the only thing
		// displayed, but it is empty.
		if (showPrimaryText && !showTitle) {
			dojo.style(this._primaryTextNode, "minHeight", "1em");
		} else {
			dojo.style(this._primaryTextNode, "minHeight", "");
		}
		
//		if ((this.showAddTextButton || this.showAddNameButton) && (titleUnderEdit || primaryTextUnderEdit)) {
//			dojo.addClass(this._addFieldAnchor, "enable");
//			dojo.addClass(this._overlayButtonContainerNode, "enable");
//			this._addFieldAnchor.innerHTML = this.showAddTextButton ? this._messages.addPrimaryText : this._messages.addName;
//		}
//		else {
			dojo.removeClass(this._addFieldAnchor, "enable");
			dojo.removeClass(this._overlayButtonContainerNode, "enable");
//		}
//		
//		// Add an adjustment to button position to compensate for non-zero margins on non-title text.
//		if (!showTitle) {
//			dojo.addClass(this._addFieldAnchor, "noTitle");
//		} else  {
//			dojo.removeClass(this._addFieldAnchor, "noTitle");
//		}
	},
	
	_toggleChildren: function (ev) {
		dojo.toggleClass(this.domNode, "collapsed");
		this.onToggleChildren();
		dojo.stopEvent(ev);
	},
	
	headingLevelChanged: function () {
		this._setHeadingMargin(this.sectionNumberWrapper, this._primaryTextNode);				
	},
	
	loadEmbeddedResources: function (isExtendedType) {
        com.ibm.rdm.web.artifactgrid.ArtifactDisplay.loadEmbeddedResources({
            isExtendedType: isExtendedType,
            baselineURI: this.baselineURI,
            textNode: this._primaryTextNode,
            displayDate: this.displayDate,
            moduleBaselineURI: this.moduleBaselineURI,
            grid: this.grid,
            resourceURI: this.resource.resourceURI,
            addDestroy: this._addDestroy
        });
		
	},
	
	_createImg: function (wrappedResourceURI, wrappedResourceRevisionURI) {
		var srcUri = wrappedResourceRevisionURI || wrappedResourceURI;
		
		// NOTE: VVC - wrappedResourceURI is passed in with accept=none and a VVC configuration already on the URI,
		// so do not add here
		
		var img = dojo.create("img", {
			src: srcUri
		});
		
		var imageReady = dojo.hitch(this, function () {
			this.grid.measureRows();
		});	
			
		img.onload = function () {
			imageReady();
		};
			
			
		if (img.complete) {
			imageReady();
		}			
		
		return img;	
	},
	
	/*
	_getWrappedResourceDisplayValue: function (resource, wrappedResourceURI, wrappedResourceRevisionURI) {
		if (MimeTypeRegistry.isImage(resource.getWrappedResourceMimeType())) {
			// Embedded Resource Loader gets run over the cell and will expand
			// out with the correct revision.
			
			wrappedResourceURI = wrappedResourceURI + "?accept=none";
			var vvcConfiguration = VvcUtils.getCurrentConfiguration();
    		if (vvcConfiguration) {
				wrappedResourceURI = wrappedResourceURI + "&" + Headers.VVC_CONFIGURATION + "=" + encodeURIComponent(vvcConfiguration.uri);
			} 
			
			// we need to add the eTag onto the wrapped resource URI (this is fixed) so updating
			// the wrapped resource does not result in a changed URI, so the browser will not fetch
			// the image.  The server ignores the eTag parameter.
			wrappedResourceURI = wrappedResourceURI + "&eTag=" + encodeURIComponent(resource.getETag());
			
			var	img = this._createImg(wrappedResourceURI);
			
			var imageReady = dojo.hitch(this, function () {
				this.grid.measureRows();
			});	
				
			img.onload = function () {
				imageReady();
			};
				
				
			if (img.complete) {
				imageReady();
			}			
			
			return img;
		} else {
			// we have a wrapped resource, create a link to it
			// Rendering the link to open the artifact requires loading the full resource.
			var retVal = this.artifactDisplay.createResourceLinkHTML(resource, false, true, resource.getTitle(), true, null, this.moduleBaselineURI, com.ibm.rdm.web.artifactgrid.ArtifactConstants.MODULE_CONTENT_COL);
			var icon = MimeTypeRegistry.getMediumImage(resource.getWrappedResourceMimeType(), this.projectURI);
			
			retVal.div.appendChild(icon);
			dojo.addClass(retVal.div, "wrappedResource");
			
			// experimental...see 52379
			if (resource.getWrappedResourceMimeType() === MimeTypes.CONTENT_TYPE_PDF) {
			
				var wrapper = this._embedCache && this._embedCache[resource.resourceURI];
				if (!wrapper || dojo.isIE) {
					var showInPlaceMsg = dojo.create("p", {
						"class": "showInPlaceMsg"
					}, retVal.div);

					var loadMe = dojo.create("a", {
						innerHTML: this._messages.showInPlace
					}, showInPlaceMsg);
					
					var onload = dojo.hitch(this, function(wrappedResource){
						
						var uri;
						if (wrappedResourceRevisionURI) {
							uri = wrappedResourceRevisionURI + "&accept=none&private&embed=true";
						} else {
							uri = wrappedResource.getWrappedResourceRevisionURI() +
							(this.baselineURI ? "&rrc:baseline=" + com.ibm.rdm.web.util.queryutils.getLastPathSegment(this.baselineURI) : "") +
							"&accept=none&private&embed=true&preventCache=" + (new Date().getTime());
						}
						
						uri += "&vvc.configuration="+encodeURIComponent(VvcUtils.getCurrentConfiguration().uri);
						
						var	that = this,
							embed = dojo.create("embed", {
								src: uri,
								type: resource.getWrappedResourceMimeType(),
								style: {
									width: "100%",
									height: "100%"
								}
							}),
							wrapper = dojo.create("div", {
								"class": "resizeHandleContainer",
								style: "height:500px;"
								}, retVal.div);
						
						dojo.empty(retVal.div);
						
						new dojox.layout.ResizeHandle({
							resizeAxis: "y",
							targetContainer: wrapper,
							activeResize: !dojo.isIE
						}).placeAt(wrapper);
						
						
						if (!this.subscribed) {
							dojo.subscribe("/dojo/resize/stop", function(inst){
							
								that.grid.measureRows();
							});
							
							this.subscribed = true;
						}
						
						wrapper.appendChild(embed);
						retVal.div.appendChild(wrapper);
						
						// 77865: IE strikes again..
						if (dojo.isIE) {
							wrapper.onresize = function() {
								dojo.empty(wrapper);
								var newEmbed = dojo.create("embed", {
									src: uri,
									type: resource.getWrappedResourceMimeType(),
									style: {
										width: "100%",
										height: "100%"
									}
								});
								wrapper.appendChild(newEmbed);
								new dojox.layout.ResizeHandle({
									resizeAxis: "y",
									targetContainer: wrapper,
									activeResize: false
								}).placeAt(wrapper);
							};
						}
						
						if (!this._embedCache) {
							this._embedCache = {};
							this._embedCache[resource.resourceURI] = wrapper;
						}
						
						that.grid.measureRows();
					});
					
					if (wrappedResourceRevisionURI) {
						//If we already have the specific revision URI, we can just use it
						dojo.connect(loadMe, "onclick", this, onload);
					} else {
						//Otherwise we need to go and load the base artifact
						dojo.connect(loadMe, "onclick", this, function(){
							ResourceManager.get({
								resourceURI: resource.resourceURI
							}, onload);
						});
					}
				} else {
					retVal.div.appendChild(wrapper);
				}
			}
			if (retVal.destroy) {
				this._addDestroy(retVal.destroy);
			}
			return retVal.div;
	    }	
		
	},*/
	
	_addDestroy: function(widgetToDestroy) {
		if (!this._widgetsToDestroy) {
			this._widgetsToDestroy = [];
		}
		this._widgetsToDestroy.push(widgetToDestroy);
	},
	
	_displayEditors: function(){
		// summary:
		//		Replace the display versions of attributes with the corresponding
		//		editors if either of the attributes we deal with is being edited.
		var attributeURI = RRMTypes.getPrimaryTextURI(this.projectURI);
		if (this.primaryTextEditable && EditManager.isUnderEdit(this.projectURI, this.artifactURI, attributeURI) && !this._destroyed) {
			this._openPrimaryTextEditor(attributeURI);
		}
//		attributeURI = RRMTypes.getWellKnownAttributeURI(this.projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title);
//		if (this.titleEditable && EditManager.isUnderEdit(this.projectURI, this.artifactURI, attributeURI)  && !this._destroyed) {
//			this._openTitleEditor(attributeURI);
//		}
	},
	
	_setHeadingMargin: function(sectionNumberElement, headingElement) {
		// summary:
		//		Sets the margin of the heading element such that it is positioned to the right of
		//		the section number.
		// sectionNumberElement:
		//		Node containing the section number.
		// headingElement:
		//		Node containing the heading text.
		dojo.style(this.domNode, "visibility", "visible");

	},
	
	_removeHeadingMargin: function(headingElement) {
		// summary:
		//		removes the effect of _setHeadingMargin, to return
		//		a node to its default state.
		// headingElement:
		//		Node containing the heading text.
		dojo.style(headingElement, "marginLeft", "");
	},
	
	_handleClick: function(event) {
		var target = event.target;
		target = DomUtils.findAncestorByAttr(target, "clickTarget");
		
		if (target === this._addFieldAnchor) {
			// A click on the button to add a missing field
			// Tell the grid that this cell is going into edit mode.
			this.grid.notifyEditStartedOnCell(this.rowKey, this.columnId);
			if (this.showAddTextButton) {
				// Add in primary text
				this._startPrimaryTextEdit();
				this._refresh();
				
			}
			else if (this.showAddNameButton) {
				// Add in title
				this._startTitleEdit();
				this._refresh();
			}
			dojo.stopEvent(event);
		}
		// We allow the event to propagate up to the module, where it may select the
		// row, or similar.
	},
	
	_startTitleEdit: function() {
		// summary:
		//		Records in the edit manager that the title attribute is to be edited.
		if (!this.titleEditable) {
			return;
		}
		var attributeURI = RRMTypes.getWellKnownAttributeURI(this.projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title);
		
		EditManager.startEdit(this.projectURI, this.artifactURI, attributeURI, this._lockFailCallback);
	},
	
	_startPrimaryTextEdit: function() {
		// summary:
		//		Records in the edit manager that the primary text attribute is to be edited.
		if (!this.primaryTextEditable) {
			return;
		}	
		var attributeURI = RRMTypes.getPrimaryTextURI(this.projectURI);
		EditManager.startEdit(this.projectURI, this.artifactURI, attributeURI, this._lockFailCallback);
	},
	
//	_openTitleEditor: function(titleAttributeURI){
//		// summary:
//		//		open an editor on the title field
//		
//		var onTitleEditorAvailable = function(editorNode) {
//			// summary:
//			//		Callback invoked when the editor is available
//			
//			if (this.isHeading) {
//				// For some reason, styles on the text area do not work if applied through CSS
//				// or through style argument when text area created - so apply here.
//				dojo.style(editorNode, { color: "#666", fontWeight: "bold" } );
//			}
//			dojo.place(editorNode, this._titleNode, "only");
//			
//			// sigh... undo the patch to stop Firefox from jittering...
//			if (dojo.isFF) {
//				setTimeout(dojo.hitch(this, function(){
//					dojo.style(this._titleNode, {
//						"overflowY": "visible",
//						"height": "auto"
//					});
//				}), 0);
//			}
//		};
//		
//		if (dojo.isFF) {
//			dojo.style(this._titleNode, {
//				"overflowY": "hidden",
//				"height": dojo.position(this._titleNode).h + "px"
//			});
//		}
//		
//		EditManager.getEditor({
//			projectURI: this.projectURI,
//			artifactURI: this.artifactURI,
//			definitionURI: titleAttributeURI,
//			callback: dojo.hitch(this, onTitleEditorAvailable),
//			resource: this.resource,
//			containingNode: this._titleNode
//		});
//	},
	
	_openPrimaryTextEditor: function(primaryTextAttributeURI){
		// summary:
		//		open an editor on the primary text field
		
		var onPrimaryTextEditorAvailable = dojo.hitch(this, function(editorNode){
            this.grid.runRowClassFunctions([this.artifactURI]);                                                 
			dojo.place(editorNode, this._primaryTextNode, "only");
		});
		
		var editorArgs = {isModuleEditor: true};
		if (this.isHeading /*&& !this.showAddTextButton*/) {
			// Primary text is being displayed as a heading.  Add classes to the
			// editor so that it is styled accordingly.
			var depth = this.depth;
			var headingLevelClass = "h" + ((depth < 6) ? depth : 6);
			editorArgs["additionalBodyClasses"] = ["heading", headingLevelClass];
		}
		
		EditManager.getEditor({
			projectURI: this.projectURI,
			artifactURI: this.artifactURI,
			lockingURI: this.lockingURI,
			definitionURI: primaryTextAttributeURI,
			callback: dojo.hitch(this, onPrimaryTextEditorAvailable),
			resource: this.resource,
			containingNode: this._primaryTextNode,
			editorArgs: editorArgs,
			editorCallbacks: this.editorCallbacks
		});
		
		this._primaryTextUnderEdit = true;
	},
	
	_setPrimaryTextEditorStyle: function() {
		// summary:
		//		Set the styling of the rich text editor to either heading or not, as closely as
		//		can be done with an active editor.
		var primaryTextURI = RRMTypes.getPrimaryTextURI(this.projectURI);
		var editor = EditManager.getExistingEditor(this.projectURI, this.artifactURI, primaryTextURI);
		// Add and remove classes so the editor is styled appropriate to the heading level.
		var classesToAdd,
			classesToRemove = ["h1", "h2", "h3", "h4", "h5", "h6"];
		
		if (this.isHeading) {
			
			// Add heading styling to the editor
			var depth = this.depth;
			var headingLevelClass = "h" + ((depth < 6) ? depth : 6);
			classesToAdd = ["heading", headingLevelClass];
			editor.updateBodyClasses(classesToRemove, classesToAdd);
			
			// Create a section number and move the twistie to it.
			if (!this.sectionNumberWrapper) {
				this.sectionNumberWrapper = dojo.create("div", {"class": headingLevelClass + " sectionNumber"}, this.domNode, "first");	
			}
			dojo.place(this._createSectionSpan(), this.sectionNumberWrapper, "only");
			dojo.place(this._twistie, this.sectionNumberWrapper);
			
			this._setHeadingMargin(this.sectionNumberWrapper, this._primaryTextNode);
			
		} else {
			// remove heading styling from the editor
			classesToRemove.push("heading");
			editor.updateBodyClasses(classesToRemove);
			
			// Move the twistie to non-heading position and remove section number
			dojo.place(this._twistie, this.domNode, "first");
			if (this.sectionNumberWrapper) {
				dojo.destroy(this.sectionNumberWrapper);
				this.sectionNumberWrapper = null;
			}
			
			this._removeHeadingMargin(this._primaryTextNode);
		}
		
		// make the editor resize its height to account for the changed style of its content.
		editor.triggerEditorResize();
		
	},
	
	_createSectionSpan: function () {
		
		if (this._sectionNumberSpan) {
			dojo.destroy(this._sectionNumberSpan);
		}
		
		this._sectionNumberSpan =  dojo.create("span", {
				innerHTML: this.sectionNumber
		});
		
		return this._sectionNumberSpan;
	}
		
});


com.ibm.rdm.web.artifactgrid.ContentCellWidget.findInNode = function (node) {
	// Summary:
	//		A utility for finding the widget instance within the argument node.  
	//		Note that should there be more than one ContentCellWidget in the supplied node,
	//		only the first is returned
	var widgetDomNode = dojo.query('div[id^="com_ibm_rdm_web_artifactgrid_ContentCellWidget"]', node)[0];
	if (widgetDomNode) {
		return dijit.byNode(widgetDomNode);
	}
};


})();
