/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
/* global define */
/* global document */
/* global setTimeout */
/* global alert */

/* Module with methods for fetching code review data */
define([
    "jquery",
    "$cache",
    "globals",
    "utils",
    "formCommon"
], function (
    $,
    $cache,
    globals,
    utils,
    formCommon
) {
    
/* --- private properties --- */
    
    // fetch globals
    var _codeReviewGlobal = globals.getCrGlobals(),
        _webServices = globals.getUrlService();
    
    // get code review metadata
    function _getCrMetadata(crId) {
        return $.ajax({
            type: "GET",
            url: _webServices.CR_METADATA_GET_SERVICE_URI,
            data : {
                itemId: crId
            },
            error: utils.ajaxErrorHandler,
            dataType: "JSON"
        });
    }

    // begin to populate code review metadata on page, then request consolidated changes report
    function _metadataJSONHandler(crData, status, xhr) {
        if (typeof crData === "undefined") return;

        // attach metadata to global
        _codeReviewGlobal.crData = crData;
        // set review state id
        formCommon.setReviewState(crData.state);
        // set review item id
        formCommon.setReviewItemId(crData.itemId);

        // request consolidated changes
        return _getConsolidatedChanges(crData);
    }

    // retrieve consolidated changes based on code review change sets
    function _getConsolidatedChanges(crData) {
        return $.ajax({
            type: "POST",
            url: _webServices.CR_CONSOLIDATED_CHANGES_SERVICE_URI,
            data: {
                changeSetIds : Object.getOwnPropertyNames(crData.closedChangeSets)
            },
            beforeSend: function(req) {
                req.setRequestHeader("Accept", "text/json");
            },
            traditional: true, // must be true so that changeSetIds array can be formatted properly
            error: utils.ajaxErrorHandler,
            dataType: "JSON"
        });
    }

    // process consolidated changes (JSON) and request for parent paths
    function _consolidatedChangesJSONHandler(json, status, xhr) {
        if (typeof json === "undefined") return;

        var filesMap = {};

        var parentPathDeferred = [];
        
        var fileStates = _codeReviewGlobal.crData.fileStates;
        var fileStartingStates = _codeReviewGlobal.crData.fileStartingStates;
        
        var reports = json["soapenv:Body"].response.returnValue.value.reports;
        for (var rptIdx = 0; rptIdx < reports.length; rptIdx++) {
	        var itemChanges = reports[rptIdx].itemChanges;
	        if (typeof itemChanges === "undefined") {
	            continue;   
	        }

	        var parentIds = [];

	        // extract file data from json
	        for (var chgIdx = 0; chgIdx < itemChanges.length; chgIdx++){
	            var itemChange = itemChanges[chgIdx];
	            var nameIndex, parentIndex;
	            if (itemChange.finalNameIndex < 0) {
	                // this means file was deleted, use the initial name/parent indices
	                nameIndex = itemChange.initialNameIndex;
	                parentIndex = itemChange.initialParentIndex;
	            } else {
	                nameIndex = itemChange.finalNameIndex;
	                parentIndex = itemChange.finalParentIndex;
	            }
	            var parentId = itemChange.parentIds[parentIndex];
	            var changes = itemChange.changes;
	            var lastChange = changes[changes.length - 1];
	            
	            var afterState = changes[0].afterState;
	            var afterStateId = (itemChange.uniqueEndingState || (afterState ? afterState.stateId : null));
	            var afterStateInfo = afterStateId ? fileStates[afterStateId] : null; // file after state info

	            var beforeState = null;
	            var beforeStateId = null;
	            
	            // set before state id and before state
	            if (fileStartingStates && fileStartingStates[itemChange.handle.itemId]) {
		            // determine whether we have a merge state. If so, set the before state id to the merge state id
	            	beforeStateId = fileStartingStates[itemChange.handle.itemId];
	            	
	            	// iterate through each item change and find the merge state which has
	            	// the matching before state id
	            	$.each(itemChange.changes, function(index, change){
	            		var merges = change.mergeStates;
	            		if (merges != null) {
		            		var lastMergeState = merges[merges.length - 1];
		            		if (lastMergeState != null && lastMergeState.stateId == beforeStateId) {
		            			beforeState = merges[merges.length - 1];
		            		}
	            		}
	            	});
	        	}

        		// merge state is not the starting state, we use last change's before state as our starting
        		// state since changes are order from latest to oldest from index 0 to the last index
	            if (beforeState == null) {
	        		beforeState = lastChange.beforeState;
	        		beforeStateId = beforeState ? beforeState.stateId : null;
	        	}
	            
	            var beforeStateInfo = beforeStateId ? fileStates[beforeStateId] : null; // file before state info
	            
	            parentIds.push(parentId);
	            filesMap[itemChange.handle.itemId] = {
	            	initialName: null, // See below
	                name: itemChange.names[nameIndex], // The name of the file
	                shortName: "",
	                pathSegments: null, // This will be an array of strings representing each item in the path. A value of 'null' indicates that the given parent is unknown. 
	                pathText: null, // Text of the path suitable for display and sorting
	                type: itemChange.handle.itemType,
	                itemId: itemChange.handle.itemId,
	                componentId: reports[rptIdx].componentId,
	                parent: itemChange.parentIds[parentIndex],
	                bStateId: beforeStateId,
	                aStateId: afterStateId,
	                bStateSize: null,
	                aStateSize: null,
	                bContentType: null,
	                aContentType: null,
	                bCharset: 'utf-8',
	                aCharset: 'utf-8',
	                forceShow: false,
	            };

	            // set before state size and hide file if contentType is unsupported or file size too large
	            if (beforeStateInfo) {
	            	filesMap[itemChange.handle.itemId].bStateSize = beforeStateInfo.size;
	            	filesMap[itemChange.handle.itemId].bContentType = beforeStateInfo.contentType;
	            	filesMap[itemChange.handle.itemId].bCharset = beforeStateInfo.encoding;
	            } 
	            
	            // set after state size and hide file if contentType is unsupported or file size too large
	            if (afterStateInfo) {
	            	filesMap[itemChange.handle.itemId].aStateSize = afterStateInfo.size;
	            	filesMap[itemChange.handle.itemId].aContentType = afterStateInfo.contentType;
	            	filesMap[itemChange.handle.itemId].aCharset = afterStateInfo.encoding;
	            }

	            if (itemChange.initialNameIndex < 0) {
	            	// This is a creation. We set the name to a value to avoid NPEs. 
	            	filesMap[itemChange.handle.itemId].initialName = itemChange.names[nameIndex];
	            }
	            else {
	            	filesMap[itemChange.handle.itemId].initialName = itemChange.names[itemChange.initialNameIndex];
	            }
	            
	            // Set the version IDs
	            if (beforeState && beforeState.versionableIdentifier && beforeState.versionableIdentifier.shortVersionId) {
	            	filesMap[itemChange.handle.itemId].bVersionNumber = beforeState.versionableIdentifier.shortVersionId;
	            }
	            
	            if (afterState && afterState.versionableIdentifier && afterState.versionableIdentifier.shortVersionId) {
	            	filesMap[itemChange.handle.itemId].aVersionNumber = afterState.versionableIdentifier.shortVersionId;
	            } 
	            
	            // Get the change type
	            filesMap[itemChange.handle.itemId].isAdd = (0x01 == itemChange.kind);
	            filesMap[itemChange.handle.itemId].isDelete = (0x10 == itemChange.kind);
	            filesMap[itemChange.handle.itemId].isMod = ((0x02 & itemChange.kind) == 0x02);
	            filesMap[itemChange.handle.itemId].isRename = ((0x04 & itemChange.kind) == 0x04);
	            filesMap[itemChange.handle.itemId].isReparent = ((0x08 & itemChange.kind) == 0x08);
	        }
	        
	        // Get the list of change sets for this component
	        var csIds = [];
	        var csInfos = reports[rptIdx].changeSetInfos;
	        for (var csIdx = 0; csIdx < csInfos.length; csIdx++) {
	        	csIds.push(csInfos[csIdx].itemId);
	        }
	        
	        parentPathDeferred.push(_getParentPathsJSON(parentIds, csIds));
	    }

        // attach files to global
        _codeReviewGlobal.filesMap = filesMap;

        if (parentPathDeferred.length == 1) {
        	// Handle the singleton case
        	return parentPathDeferred[0].then(function (r) {
        		var ret = {};
        		
        		ret[r["soapenv:Body"].response.returnValue.value.componentId] = r["soapenv:Body"].response.returnValue.value.parentPathInfos

        		return ret;
        	});
        }
        
        return $.when.apply($, parentPathDeferred).then(function () {
        	// Merge the per-component responses into a single data structure
        	var args = {};
        	for (var i = 0; i < arguments.length; i++) {
        		if (arguments[i][1] == "success") {
        			var r = arguments[i][0]["soapenv:Body"].response.returnValue.value;
        			args[r.componentId] = r.parentPathInfos;
        		}
        	}
        	
        	return args;
        });
    }

    // use service to get parent paths for changed files
    function _getParentPathsJSON(parentIds, csIds) {
        var crData = _codeReviewGlobal.crData;
        return $.ajax({
            type: "POST",
            url: _webServices.CR_CONSOLIDATED_PARENT_PATHS_SERVICE_URI,
            data: {
                contextItemNamespace : "com.ibm.team.scm",
                contextItemType : "Workspace",
                contextItemId : crData.stream.id,
                changeSetIds : csIds,
                parentIds : parentIds
            },
            beforeSend: function(req){
                req.setRequestHeader("Accept", "text/json");  
            },
            traditional: true, // must be true so that changeSetIds array can be formatted properly
            error: utils.ajaxErrorHandler,
            dataType: "JSON",
        });
    }
    
    function _splitPathFromServer(path) {
    	// This path should start with a '/'
    	if (path[0] == '/') {
    		path = path.substr(1);
    	}
    	
    	return path.split('/');
    }

    // compute parent paths of changed files & sort files by full path name
    function _consolidatedParentPathsJSONHandler(consolidatedResults, status, xhr) {
        if (typeof consolidatedResults === "undefined")
            return;

        var filesMap = _codeReviewGlobal.filesMap;
        
        // Group files by component
        var filesByComponent = {};
        
        for (var id in filesMap) {
        	var compId = filesMap[id].componentId;
        	
        	if (!(compId in filesByComponent)) {
        		filesByComponent[compId] = {};
        	}
        	
            filesByComponent[compId][id] = filesMap[id];
        }

        var compsByRootFolder = {};
        for (var compId in _codeReviewGlobal.crData.components) {
        	compsByRootFolder[_codeReviewGlobal.crData.components[compId].root] = _codeReviewGlobal.crData.components[compId];
        }
        
        // Walk the files within each component and set their paths
        for (var compId in filesByComponent) {
        	var r = consolidatedResults[compId];
        	
        	// Index the consolidated results by file id
        	var paths = {};
        	for (var i = 0; i < r.length; i++) {
        		paths[r[i].itemId] = r[i]; 
        	}
        	
        	var missingPath = [];
        	
        	// Record the paths for the consolidated results
        	var files = filesByComponent[compId];
        	for (var fileId in files) {
        		var parentId = files[fileId].parent;
        		if (parentId in paths) {
            		var parent = paths[parentId];
        			if (parent.bestPathIndex != -1) {
        				var segments = _splitPathFromServer(parent.possiblePaths[parent.bestPathIndex]);
        				segments.push(files[fileId].name);
        				
        				files[fileId].pathSegments = segments;
        				continue;
        			}
        		}
        		
        		missingPath.push(fileId);
        	}
        	
        	if (missingPath.length == 0) {
        		// All of the paths in this component have been resolved.
        		continue;
        	}
        	
        	// We have one or more missing paths. See if we can construct 
        	// a partial path based on the changes.
        	var parents = {};
        	
        	for (var fileId in files) {
        		parents[fileId] = files[fileId].parent;
        	}
        	
        	var pathStrings = {};
        	for (var i = 0; i < missingPath.length; i++) {
        		var path = [];
        		
        		var parentId = missingPath[i];
        		while (files[parentId]) {
        			path.unshift(files[parentId].name);
        			
        			parentId = files[parentId].parent;
        		}
        		
        		if (!(parentId in compsByRootFolder)) {
        			path.unshift(null);
        		}
        		
        		files[missingPath[i]].pathSegments = path;
        	}
        }
        
        for (var id in filesMap) {
        	filesMap[id].pathText = filesMap[id].pathSegments.join('/');
        }
    }
    
/* --- public properties --- */
    
    // perform initial chain of requests to get full code review data
    // - also takes a callback method which will be executed upon success
    function fetchCodeReviewData(callback) {
        var crId = utils.getUrlParameter("itemId");
        if (crId !== false) {
            _getCrMetadata(crId)
                .then(_metadataJSONHandler)
                .then(_consolidatedChangesJSONHandler)
                .then(function (consolidatedResults, status, xhr) {
                    if (typeof consolidatedResults === "undefined") return;
                    
                    // attach parent paths to file names
                    _consolidatedParentPathsJSONHandler(consolidatedResults, status, xhr);

                    // data fetch done at this point

                    // execute any callbacks after data fetch is complete
                    if (callback) callback();
                });
        }
    }
    
    // return module
    return {
        fetchCodeReviewData : fetchCodeReviewData
    };
    
});